name = 'sour-cereal'
version = '1.0.8'

author = 'Carlos Alberto Duarte Pinto'
author_email = 'carlos.adpinto@gmail.com'

keywords = ['abstraction', 'data extraction', 'data sources', 'interface',
            'etl']

description = 'An abstraction of data source for extraction applications usage'

url = 'https://github.com/CarlosAdp/sour-cereal'

download_url = 'https://github.com/CarlosAdp/sour-cereal/archive/' +\
        f'v{version}.tar.gz'

classifiers = [
    'Programming Language :: Python :: 3.6',
    'Programming Language :: Python :: 3.7',
    'Programming Language :: Python :: 3.8',
    'Operating System :: OS Independent',
    'Topic :: Utilities',
    'License :: OSI Approved :: MIT License',
    'Intended Audience :: Developers'
]

license = 'MIT'
